<?php
namespace apexl\Io\modules\display\components;

use apexl\Io\modules\component\entities\componentEntity;

class FileButton extends componentEntity {
    public function __construct()
    {
        parent::__construct();
        $this->addRoute('/');
        $this->addText( '');
    }

    public function addRoute($route){
        $this->addProperty('route', $route);
        return $this;
    }

    public function addText($text){
        $this->addProperty('text', $text);
        return $this;
    }

    public function addIcon($icon){
        $this->addProperty('icon', $icon);
        return $this;
    }

    public function disabled($disabled = FALSE){
        $this->addProperty('disabled', $disabled);
        return $this;
    }
    
}